package org.ajc.core;

import org.aspectj.tools.ant.taskdefs.AjcTask;

import protoj.lang.CompileFeature;
import protoj.lang.StandardProject;
import protoj.util.ArgRunnable;

// ajc shell script compiles and invokes this main class.
public final class AjcCore {
	public static void main(String[] args) {
		AjcCore core = new AjcCore(args);

		// process the commands captured from the command line
		core.dispatchCommands();
	}

	private StandardProject project;

	public AjcCore(String[] args) {
		project = new StandardProject(args, "ajc Version 1.0");

		// enables configuration support with no resolving of ${x} placeholders
		// so that files from conf/profile get merged to the project root tree
		project.initDirConfig();

		// project resources can access the project code with velocity markup -
		// eg the maven and ivy files don't have to use hard-coded values
		project.getResourceFeature().getContext().put("ajc", this);

		// the specified ivy.xml file will be use for downloading dependencies
		project.getRetrieveFeature().initIvy("/ajc/ivy.xml");

		// load time weaving is enabled for this project
		project.getDispatchFeature().initLoadTimeWeaving(null);

		// the ajc compiler is used to compile the entire project
		project.getCompileFeature().initConfig(true,
				new ArgRunnable<CompileFeature>() {
					public void run(CompileFeature feature) {
						AjcTask compileTask = feature.getAjcCompileTask()
								.getCompileTask();
						compileTask.setMaxmem("16m");
						compileTask.setSource("1.5");
					}
				});

		// a new vm is used to run all commands with the specified main class
		project.getDispatchFeature().initBootstrap("org.ajc.system.AjcProject",
				"16m");
	}

	/**
	 * This method is accessed from inside the ivy.xml and pom.xml resources,
	 * using velocity markup, instead of hard-coding the value.
	 * 
	 * @return
	 */
	public String getAjcVersion() {
		return "1.0";
	}

	/**
	 * This method is accessed from inside the ivy.xml and pom.xml resources,
	 * using velocity markup, instead of hard-coding the value.
	 * 
	 * @return
	 */
	public String getJunitVersion() {
		return "4.4";
	}

	/**
	 * The main ProtoJ pojo, reused by the AjcProject class.
	 * 
	 * @return
	 */
	public StandardProject getProject() {
		return project;
	}

	/**
	 * Dispatches command line arguments to the commands that were set up in the
	 * constructors of this class and of AjcProject.
	 */
	public void dispatchCommands() {
		getProject().getDispatchFeature().dispatchCommands();
	}

}
